#include <stdio.h>
#include <stdlib.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <sdcard/wiisd_io.h>
#include <fat.h>
#include <time.h>
#include <SDL.h>

void drawdot(void *xfb, GXRModeObj *rmode, float fx, float fy, float width, float heigth, u32 color) {
	u32 *fb;
	int px,py;
	int x,y;
	fb = (u32*)xfb;

	y = fy * rmode->xfbHeight / rmode->xfbHeight;
	x = fx * rmode->fbWidth / rmode->fbWidth / 2;
	
	for(py=y; py<=(y+heigth); py++) {
		if(py < 0 || py >= rmode->xfbHeight)
				continue;
		for(px=x; px<=(x+width/2); px++) {
			if(px < 0 || px >= rmode->fbWidth/2)
				continue;
			fb[rmode->fbWidth/VI_DISPLAY_PIX_SZ*py + px] = color;
		}
	}
}

struct Rect
{
	short x;
	short y;
	short width;
	short height;
};

bool Intersect(Rect* rect1, Rect* rect2)
{
	if(rect1->y+rect1->height < rect2->y)		return false;
	if(rect1->y > rect2->y+rect2->height)		return false;
	if(rect1->x+rect1->width < rect2->x)		return false;
	if(rect1->x > rect2->x+rect2->width)		return false;

	return true;
}


Rect CreateNewTarget(GXRModeObj *rmode)
{
	Rect t;
	t.width = 50 + rand() % 60;
	t.height = 50 + rand() % 60;
	t.x = rand() % (rmode->fbWidth - t.width - 200) + 100;
	t.y = rand() % (rmode->xfbHeight - t.height - 200) + 100;
	return t;
}

//---------------------------------------------------------------------------------
int main(int argc, char **argv) 
{
	int fbi = 0;
	void *xfb[2];
	GXRModeObj *rmode = NULL;

	// Initialise the video system
	VIDEO_Init();
	rmode = VIDEO_GetPreferredMode(NULL);
	xfb[0] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	xfb[1] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

	// This function initialises the attached controllers
	WPAD_Init();
	
	int points = 0;
	double timeLeft = 30;
	Rect currentTarget = CreateNewTarget(rmode);
	time_t lastTime;
	time(&lastTime);
	
	while(1)
	{
		console_init(xfb[fbi],20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
				
		// Call WPAD_ScanPads each loop, this reads the latest controller states
		WPAD_ScanPads();

		u32 pressed = WPAD_ButtonsDown(0);
		if ( pressed & WPAD_BUTTON_HOME ) exit(0);
		pressed = WPAD_ButtonsDown(WPAD_BALANCE_BOARD);
		if ( pressed & WPAD_BUTTON_A ) timeLeft = 30;
		
		//WPAD_IR(WPAD_CHAN_0, &ir);
		WPADData* data = WPAD_Data(WPAD_BALANCE_BOARD);
		int x = rmode->fbWidth / 2 + data->exp.wb.x * 2;
		int y = rmode->xfbHeight / 2 + data->exp.wb.y * 2;
		
		Rect player;
		player.x = x - 6;
		player.y = y - 6;
		player.width = 12;
		player.height = 12;
		
		if(timeLeft>0)
		{
			if(Intersect(&player, &currentTarget))
			{
				points++;
				timeLeft+=0.5;
				if(timeLeft>60)
				{
					timeLeft=60;
				}
				currentTarget = CreateNewTarget(rmode);
			}
		}		
		time_t now;
		time(&now);
		timeLeft -= difftime(now, lastTime);
		lastTime = now;

		if(timeLeft>0)
		{
			drawdot(xfb[fbi], rmode, 20, 60, rmode->fbWidth*timeLeft/60-20, 20, (timeLeft<10) ? COLOR_RED : COLOR_GREEN);
			drawdot(xfb[fbi], rmode, currentTarget.x, currentTarget.y, currentTarget.width, currentTarget.height, COLOR_BLUE);
		}
		drawdot(xfb[fbi], rmode, player.x, player.y, player.width, player.height, COLOR_RED);
		printf("\x1b[2;0H Points: %d  -  %d:%d", points, x, y);
		
		VIDEO_SetNextFramebuffer(xfb[fbi]);
		VIDEO_Flush();
		VIDEO_WaitVSync();
		fbi ^= 1;
	}

	return 0;
}

